/** @file simple_cd.cpp
 *  @brief OpenDynamicEngine𗘗pȈՏՓ˔NXCvg
 *
 *  @author yasufuku
 *  @date 2011/01
 *  @version 0.10
 *
 *  Copyright(C) 2011 Kensuke Yasufuku
 */
#include "simple_cd.h"
#include "sys_config.h"
#include "walk_sim.h"

static const float ode_dt = 1.0f/60.0f;
static const float ode_g  = -9.8f;
extern SystemConfig sc;

//===============================================================
// SimpleCDNXCvg
//===============================================================
//---------------------------------------------------------------
// RXgN^
//---------------------------------------------------------------
SimpleCD::SimpleCD()
{
	m_fImpulse1     = 0.0f;
	m_fImpulse2     = 0.0f;
	m_fSphereRadius = 1.0f;
	m_fFPSRatio     = 1.0f/((float)sc.nFPS*ode_dt);
	m_bGround       = false;
	m_fCollisionHeight = 0.0f;
	m_fEyeHeight = 0.0f;
}

//---------------------------------------------------------------
// fXgN^
//---------------------------------------------------------------
SimpleCD::~SimpleCD()
{
}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
int SimpleCD::Init( WalkthroughInfo * pInfo )
{
	m_fImpulse1     = pInfo->fImpulse1 * m_fFPSRatio;
	m_fImpulse2     = pInfo->fImpulse2 * m_fFPSRatio;
	m_fSphereRadius = pInfo->radius;
	m_fCollisionHeight = pInfo->eyeHeight;
	m_fEyeHeight = pInfo->eyeHeight;
	
	InitODE();
	MakeCollisionData( pInfo );

	return 0;
}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
void SimpleCD::Destroy()
{
	dWorldDestroy(m_odeWorld);
	dSpaceDestroy(m_odeSpace);
}

//---------------------------------------------------------------
// ̈ʒu擾
//---------------------------------------------------------------
const Vector & SimpleCD::GetPoint()
{
	static Vector pos;
	const double * t = dBodyGetPosition( m_odeSphereBody );
	pos.x = (float)t[1];
	pos.y = (float)t[2]+(m_fCollisionHeight - m_fSphereRadius);
	pos.z = (float)t[0];

	return pos;
}
const Vector & SimpleCD::GetFootPoint()
{
	static Vector pos;
	const double * t = dBodyGetPosition( m_odeSphereBody );
	pos.x = (float)t[1];
	pos.y = (float)t[2];
	pos.z = (float)t[0];

	return pos;
}

//---------------------------------------------------------------
// O̓Zbg
//---------------------------------------------------------------
void SimpleCD::ResetForce()
{
	m_vForce.Set();
	m_vTorque.Set();
}

//---------------------------------------------------------------
// 𓮂
//---------------------------------------------------------------
void SimpleCD::MoveForward( float sphereDir )
{
	m_vForce.x += cos( sphereDir ) * m_fImpulse1;
	m_vForce.z += sin( sphereDir ) * m_fImpulse1;
}
void SimpleCD::MoveBackward( float sphereDir )
{
	m_vForce.x += cos( sphereDir ) *  -m_fImpulse1;
	m_vForce.z += sin( sphereDir ) *  -m_fImpulse1;
}

void SimpleCD::MoveToRight( float sphereDir )
{
	m_vForce.x += cos( sphereDir + PI*0.5f ) * m_fImpulse1;
	m_vForce.z += sin( sphereDir + PI*0.5f ) * m_fImpulse1;
}
void SimpleCD::MoveToLeft( float sphereDir )
{
	m_vForce.x += cos( sphereDir - PI*0.5f ) * m_fImpulse1;
	m_vForce.z += sin( sphereDir - PI*0.5f ) * m_fImpulse1;
}

void SimpleCD::Rise()
{
	m_vForce.y = m_fImpulse2;
	if( m_fCollisionHeight > 0.0f ) m_fCollisionHeight -= 0.02f;
	else m_fCollisionHeight = 0.0f;
}

void SimpleCD::Fall()
{
	if( m_fCollisionHeight < m_fEyeHeight ) m_fCollisionHeight += 0.04f;
	else m_fCollisionHeight = m_fEyeHeight;
}

//---------------------------------------------------------------
// XV
//---------------------------------------------------------------
void SimpleCD::Update()
{
	//-----------------------------------------------------
	// CRvZ
	// ő呬x200km/hxtZ
	//-----------------------------------------------------
	const double * v = dBodyGetLinearVel( m_odeSphereBody );
	Vector lvel( (float)v[1], (float)v[2], (float)v[0]  );

	float k = 3.0f;
	if( m_bGround ) {
		k = 10.0f;
	}
	lvel.x = k * fabs(lvel.x) * lvel.x;
	lvel.y = k * fabs(lvel.y) * lvel.y;
	lvel.z = k * fabs(lvel.z) * lvel.z;
	m_vForce  = m_vForce  - lvel;

	//-----------------------------------------------------
	// O͍XV
	//-----------------------------------------------------
	dBodySetForce( m_odeSphereBody, m_vForce.z,  m_vForce.x,  m_vForce.y  );

	//-----------------------------------------------------
	// Փˌo֐
	//-----------------------------------------------------
	m_bGround = false;
	dSpaceCollide( m_odeSpace, this, &NearCallback );
	dWorldStep( m_odeWorld, ode_dt );
	
	static float timer = 0.0f;
	if( m_bGround ) {
		timer = 5.0f/m_fFPSRatio;
	} else {
		if( timer > 0.0f ) {
			timer -= 1.0f;
			m_bGround = true;
		}
	}	
	dJointGroupEmpty( m_odeContactGroup ); // WCgO[vɂ
}

//---------------------------------------------------------------
// ՓˎɌĂ΂
//---------------------------------------------------------------
void SimpleCD::NearCallback( void * data, dGeomID o1, dGeomID o2 ) 
{
	SimpleCD * p = (SimpleCD *)data;
	static const int N = 10; // ڐG_̍ől
	dContact contact[N];     // ڐG_

	// Փˏ̐ n͏Փ˓_
	int n =  dCollide( o1, o2, N, &contact[0].geom, sizeof(dContact) );

	for (int i = 0; i < n; i++) {
		contact[i].surface.mode = dContactBounce | dContactSoftCFM | dContactSoftERP;
		contact[i].surface.bounce = 0.1;          // W(0.01.0)  
		contact[i].surface.bounce_vel = 0.1;      // ɕKvȍŒᑬx
		contact[i].surface.mu = 100.0f;
		contact[i].surface.soft_cfm = 0.00005;
		contact[i].surface.soft_erp = 0.2;

		// ڐGWCg̐                         
		dJointID c = dJointCreateContact( p->m_odeWorld,
										  p->m_odeContactGroup,
										  &contact[i] ); 
		// ڐGĂQ̍̂ڐGWCgɂS
		dJointAttach( c,
					  dGeomGetBody( contact[i].geom.g1 ),
					  dGeomGetBody( contact[i].geom.g2 ) );

		//dBodySetLinearVel( dGeomGetBody(contact[i].geom.g1), 0.0,0.0,0.0 );
		if( dGeomGetBody(contact[i].geom.g1) ) {
			dBodySetAngularVel( dGeomGetBody(contact[i].geom.g1), 0.0,0.0,0.0 );
		}
		if( dGeomGetBody(contact[i].geom.g2) ) {
			dBodySetAngularVel( dGeomGetBody(contact[i].geom.g2), 0.0,0.0,0.0 );
		}
		p->m_bGround = true;
    }   
}

//---------------------------------------------------------------
// ODE
//---------------------------------------------------------------
int SimpleCD::InitODE()
{
	//--------------------------------------------------------
	// ODEݒ
	//--------------------------------------------------------
	m_odeWorld		  = dWorldCreate();
	m_odeSpace		  = dHashSpaceCreate(0);
	m_odeContactGroup = dJointGroupCreate(0);
	dWorldSetGravity( m_odeWorld, 0, 0, ode_g*m_fFPSRatio );
	
	return 0;
}

//---------------------------------------------------------------
// Փ˃f[^
//---------------------------------------------------------------
int SimpleCD::MakeCollisionData( WalkthroughInfo * pInfo )
{
	//------------------------------------------------------
	// ODE|SbV
	//------------------------------------------------------
	int i,j;
	dMatrix3 R;
	for( i=0; i<pInfo->nArchModel; i++ ) {
		if( ! pInfo->pArchModel[i].pColModel ) continue;

		MeshData * pMesh = pInfo->pArchModel[i].pColModel->pMesh;
		s16           sCluster = pMesh->sCluster;
		ClusterData * pCluster = pMesh->pCluster;
	
		for( j=0; j<sCluster; j++ ) {
			m_odeBuildingTriData = dGeomTriMeshDataCreate();
			dGeomTriMeshDataBuildSingle( m_odeBuildingTriData,
									 (float *)&pCluster[j].pVtxBuffer[5],
									 9 * sizeof(float), 
									 pCluster[j].nVtxBuffer,
									 pCluster[j].pIdxBuffer,
									 pCluster[j].nIdxBuffer,
									 3 * sizeof(int) );
			m_odeBuilding = dCreateTriMesh( m_odeSpace,
											m_odeBuildingTriData,
											0, 0, 0 );
			dGeomSetPosition( m_odeBuilding, 0, 0, 0 );
			dRFromAxisAndAngle( R, 1, 0, 0, PI*0.5);
			dGeomSetRotation( m_odeBuilding, R );
		}
	}

	//------------------------------------------------------
	// ODE l()
	//------------------------------------------------------
	dMass m;
	m_odeSphereBody = dBodyCreate( m_odeWorld );
	dMassSetZero( &m );
	dMassSetSphereTotal( &m, 50.0, pInfo->radius );
	dBodySetMass( m_odeSphereBody, &m );
	dBodySetPosition( m_odeSphereBody, pInfo->pos.z, pInfo->pos.x, pInfo->pos.y  );
	m_odeSphereGeom = dCreateSphere( m_odeSpace, pInfo->radius );
	dGeomSetBody( m_odeSphereGeom, m_odeSphereBody );
	
	return 0;
}
