/** @file walk_sim.cpp
 *  @brief EH[NX[NXCvg
 *
 *  @author yasufuku
 *  @date 2011/01
 *  @version 0.10
 *
 *  Copyright(C) 2011 Kensuke Yasufuku
 */
#include "walk_sim.h"
#include "sys_config.h"
#include "mouse.h"
#include "keyboard.h"

//===============================================================
// OQƃf[^
//===============================================================
extern SystemConfig    sc;
extern WalkthroughInfo info[];
extern ModelData       sphere_model;
extern ModelData       lady_model;
extern MotionData      lady_dance_motion;
extern ModelData       skydome_model;

//===============================================================
// 萔
//===============================================================
static const char * s_pszHideHelp = "Press F1 for help";
static const char * s_pszHelp =
"Controls:\n\tMove forward and backward with 'W' and 'S'\n\tMove left and right with 'A' and 'D'\n\tMove the mouse to rotate the camera\n\tFly with SPACE key\n\tQuit:ESC\tF1:Hide help";

//===============================================================
// WalkSimNXCvg
//===============================================================
//---------------------------------------------------------------
// 
//---------------------------------------------------------------
Task * WalkSim::Create( Task * pParent, int i )
{
	WalkSim * p = NEW WalkSim(i);
	p->Init( pParent, 0 );
	return (Task *)p;
}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
Task * WalkSim::Create( Task * pParent )
{
	WalkSim * p = NEW WalkSim(0);
	p->Init( pParent, 0 );
	return (Task *)p;
}

//---------------------------------------------------------------
// RXgN^
//---------------------------------------------------------------
WalkSim::WalkSim( int i )
{
	SetName("WalkSim");

	m_pInfo = &info[i];
	
	m_fDist    = 5.0f;
	m_fTheta   = 0.0f;
	m_fPhi     = 5.0f;
	m_pScene   = NULL;
	m_pCamera  = NULL;
	m_pLight   = NULL;
	m_pFPerson = NULL;
	m_pLady    = NULL;
	m_bGround  = true;

	m_pScene2D  = NULL;
	m_pCamera2D = NULL;
	m_pHelp     = NULL;
	m_bDispHelp = false;
	
	m_nCameraMode = 1;
	m_fFPSRatio = 60.0f/(float)sc.nFPS;
	m_fNeckPhi = 0.0f;
	m_fNeckTheta = 0.0f;
	m_fSphereDir = m_pInfo->ry;
}

//---------------------------------------------------------------
// fXgN^
//---------------------------------------------------------------
WalkSim::~WalkSim()
{
}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
int WalkSim::OnInit()
{
	//--------------------------------------------------------
	// Փ˔
	//--------------------------------------------------------
	m_CD.Init( m_pInfo );
	
	//--------------------------------------------------------
	// X^WI
	//--------------------------------------------------------
	m_pScene = Scene::Create( View::GetRoot() );

	//--------------------------------------------------------
	// J
	//--------------------------------------------------------
	Vector pos, at;
	at.x  = 0.0f;
	at.y  = 0.0f;
	at.z  = 0.0f;
	float distxz = m_fDist * cos( DegToRad(m_fPhi) );
	pos.x = distxz * cos( DegToRad(m_fTheta) ) + at.x;
	pos.y = m_fDist* sin( DegToRad(m_fPhi  ) ) + at.y;
	pos.z = distxz * sin( DegToRad(m_fTheta) ) + at.z;

	m_pCamera = Camera::Create( m_pScene, pos, at );
	m_pCamera->SetFovy( 50.0f );
	
	//--------------------------------------------------------
	// Cg
	//--------------------------------------------------------
	m_pLight = Light::Create( m_pScene );
	m_pLight->SetDirection( Vector( 2.0f, 4.0f, 3.0f ) );
	m_pLight->Intensity( Color(0.7f, 0.7f, 0.85f) );

	//--------------------------------------------------------
	// f
	//--------------------------------------------------------
	// SkyDome
	Model * pSky = Model::Create( m_pScene, &skydome_model );

	// Lady()
	m_pLady   = Model::Create( m_pScene, &lady_model );
	m_pLady->t.x = 2.4f;
	m_pLady->t.y = 3.2f;
	m_pLady->t.z = 12.3f;
	m_pLady->SetMotion( &lady_dance_motion );

	// f
	for( int i=0; i<m_pInfo->nArchModel; i++ ) {
		if( m_pInfo->pArchModel[i].pDspModel ) {
			Model * pModel = Model::Create( m_pScene, m_pInfo->pArchModel[i].pDspModel );
			pModel->r.y = PI*-0.5f;
		}
	}

	// l̎_̈ʒuij
	m_pFPerson = Model::Create( m_pScene, &sphere_model );
	m_pFPerson->t.x = m_pInfo->pos.x;
	m_pFPerson->t.y = m_pInfo->pos.y;
	m_pFPerson->t.z = m_pInfo->pos.z;
	m_pFPerson->s *= m_pInfo->radius;

	//--------------------------------------------------------
	// FPS}EXL
	//--------------------------------------------------------
	Mouse::SetFPSMode(true);

	//--------------------------------------------------------
	// Help\
	//--------------------------------------------------------
	m_pScene2D  = Scene::Create( View::GetRoot() );
	m_pCamera2D = Camera::Create( m_pScene2D );
	m_pHelp     = Font::Create( m_pScene2D );
	m_pHelp->SetColor( Color( 1.0f,1.0f,1.0f) );
	m_pHelp->SetSize(8,16);
	m_pHelp->Print( -630, 340, s_pszHideHelp );
	return TR_OK;
}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
int WalkSim::OnKill()
{
	Mouse::SetFPSMode(false);
	m_pScene->Destroy();
	m_pScene2D->Destroy();
	m_CD.Destroy();

	return TR_OK;
}

//---------------------------------------------------------------
// C
//---------------------------------------------------------------
int WalkSim::OnMain()
{
	// _𓮂
	MovePerson();

	// Փ˔菈
	m_CD.Update();
	
	// Lady[V
	if( m_pLady ) m_pLady->IncTime(0.5f);

	// _̍XV
	m_pFPerson->t = m_CD.GetFootPoint();

	// JXV
	// J؂ւ
	if( Keyboard::Trigger( KEY_C ) ) {
		m_nCameraMode = 0x01 & ~m_nCameraMode;
		if( m_nCameraMode ) Mouse::SetFPSMode(true);
		else                Mouse::SetFPSMode(false);
	}

	if( m_nCameraMode ) {
		UpdateFPCamera();
	} else {
		UpdateTPCamera();
	}

	//printf("DEBUG(%f,%f,%f)\n", m_pPerson->t.x, m_pPerson->t.y, m_pPerson->t.z);

	if( Keyboard::Trigger(KEY_F1) ) {
		if( m_bDispHelp ) {
			m_pHelp->Print( -630, 340, s_pszHideHelp );
			m_bDispHelp = false;
		} else {
			m_pHelp->Print( -630, 250, s_pszHelp );
			m_bDispHelp = true;
		}
	}

	return TR_OK;
}

//---------------------------------------------------------------
// ړ
//---------------------------------------------------------------
int WalkSim::MovePerson()
{
	// O͌vZ
	m_CD.ResetForce();

	//-----------------------------------------------------------
	// WADSL[œ
	//-----------------------------------------------------------
	if( Keyboard::Press( KEY_D) ) {
		m_CD.MoveToRight(m_fSphereDir);
	}
	if( Keyboard::Press( KEY_A) ) {
		m_CD.MoveToLeft(m_fSphereDir);
	}
	
	if( Keyboard::Press( KEY_S) ) {
		m_CD.MoveBackward(m_fSphereDir);
	}
	if( Keyboard::Press( KEY_W) ) {
		m_CD.MoveForward(m_fSphereDir);
	}
	
	//-----------------------------------------------------------
	// L[œ
	//-----------------------------------------------------------
	static float acl = 0.0f;
	static const float pdl = 0.003f;
	static const float max = 0.02f;

	if(  Keyboard::Press( KEY_LEFT ) | Keyboard::Press( KEY_RIGHT) ) {
		if( Keyboard::Press( KEY_RIGHT) ) {
			if( Keyboard::Press( KEY_SHIFT ) ) {
				m_CD.MoveToRight(m_fSphereDir);
			} else {
				if( acl <  max ) acl += pdl;
				m_fNeckPhi *= 0.9f;
			}
		}
		if( Keyboard::Press( KEY_LEFT ) ) {
			if( Keyboard::Press( KEY_SHIFT ) ) {
				m_CD.MoveToLeft(m_fSphereDir);
			} else {
				if( acl > -max ) acl -= pdl;
				m_fNeckPhi *= 0.9f;
			}
		}
	} else {
		acl *= 0.6f;
	}
	
	if( Keyboard::Press( KEY_UP   ) ) {
		m_CD.MoveForward(m_fSphereDir);
		m_fNeckPhi *= 0.9f;
	}
	if( Keyboard::Press( KEY_DOWN ) ) {
		m_CD.MoveBackward(m_fSphereDir);
		m_fNeckPhi *= 0.9f;
	}

	m_fSphereDir +=  acl * m_fFPSRatio;
	
	//-----------------------------------------------------------
	// }EXœ
	//-----------------------------------------------------------
	int mx, my;
	static const float limit_angle = 88.0f;
	static const float sensitive = 0.03f;

	if( Mouse::Drag( MOUSE_L_BUTTON  ) ) {
		Mouse::GetDrag( mx, my );
	} else {
		Mouse::GetMove( mx, my );
	}
	// }EXړʂɂU
	m_fNeckTheta += (float)mx * sensitive;
	if( my > 0.0f ) {
		if( m_fNeckPhi <  limit_angle ) m_fNeckPhi += (float)my*sensitive;
	} else {
		if( m_fNeckPhi > -limit_angle ) m_fNeckPhi += (float)my*sensitive;
	}
	m_fSphereDir += DegToRad(m_fNeckTheta);
	m_fNeckTheta *= 0.5f;

	if( m_nCameraMode ) {
		if( Mouse::Click( MOUSE_R_BUTTON  ) ) {
			m_CD.MoveBackward(m_fSphereDir);
		}
		if( Mouse::Click( MOUSE_L_BUTTON  ) ) {
			m_CD.MoveForward(m_fSphereDir);
		}
	}
	
	//-----------------------------------------------------------
	// Wv
	//-----------------------------------------------------------
	if( Keyboard::Press( KEY_SPACE ) || ( Mouse::Drag(MOUSE_L_BUTTON) && Mouse::Drag(MOUSE_R_BUTTON) )) {
		m_CD.Rise();
	} else {
		m_CD.Fall();
	}

	return 0;
}

//---------------------------------------------------------------
// 1l̃JXVXV
//---------------------------------------------------------------
int WalkSim::UpdateFPCamera()
{
	m_pFPerson->SetDisp(false);
	m_pCamera->t = m_CD.GetPoint();
	m_pCamera->lookat.x = m_pCamera->t.x + cos(m_fSphereDir + DegToRad(m_fNeckTheta) );
	m_pCamera->lookat.y = m_pCamera->t.y - 2.0f * sin( DegToRad(m_fNeckPhi) );
	m_pCamera->lookat.z = m_pCamera->t.z + sin(m_fSphereDir + DegToRad(m_fNeckTheta) );

	return 0;
}

//---------------------------------------------------------------
// 3l̃JXVXV
//---------------------------------------------------------------
int WalkSim::UpdateTPCamera()
{
	int vx, vy;
	m_pFPerson->SetDisp(true);
	if( Mouse::Drag( MOUSE_L_BUTTON ) ) {
		Mouse::GetDrag( vx, vy );
		m_fTheta += (float)vx * 0.5f;
		if( vy > 0.0f ) {
			if( m_fPhi <  80.0 ) m_fPhi += (float)vy*0.2f;
		} else {
			if( m_fPhi > -80.0 ) m_fPhi += (float)vy*0.2f;
		}
	}

	if( Mouse::Drag( MOUSE_M_BUTTON ) ) {
		Mouse::GetDrag( vx, vy );
		m_fDist += (float)vx * -0.3f;		
		if( m_fDist < 0.1f ) {
			m_fDist = 0.1f;
		}
	}

	if( Mouse::Drag( MOUSE_R_BUTTON ) ) {
		Mouse::GetDrag( vx, vy );
		m_pCamera->lookat.x += (float)vx * cos( DegToRad(m_fTheta+90) ) * 0.2f;
		m_pCamera->lookat.z += (float)vx * sin( DegToRad(m_fTheta+90) ) * 0.2f;
		m_pCamera->lookat.y += (float)vy*0.3f;
	}
	m_pCamera->lookat = m_CD.GetPoint();

	float distxz = m_fDist * cos( DegToRad(m_fPhi) );	
	m_pCamera->t.x = distxz * cos( DegToRad(m_fTheta) ) + m_pCamera->lookat.x;
	m_pCamera->t.y = m_fDist* sin( DegToRad(m_fPhi  ) ) + m_pCamera->lookat.y;
	m_pCamera->t.z = distxz * sin( DegToRad(m_fTheta) ) + m_pCamera->lookat.z;

	return 0;
}
