/** @file handle.h
 *  @brief HandleNXwb_
 *
 *  @author yasufuku
 *  @date 2004/11
 *  @version 0.10
 *
 *  Copyright(C) 2004 Kensuke Yasufuku
 */
#ifndef _HANDLE_H_
#define _HANDLE_H_

#include "include.h"

/**
 *	  @brief HandleNX
 *
 *    HandleNX
 *		GAME PROGRAMMING GEMSQlɂ܂
 */
template <typename TAG>
class Handle
{
private:
	enum {
		// sizes to use for bit fields
		MAX_BITS_INDEX = 16,
		MAX_BITS_MAGIC = 16,
			
		// sizes to compare against for asserting deferences
		MAX_INDEX = ( 1 << MAX_BITS_INDEX ) - 1,
		MAX_MAGIC = ( 1 << MAX_BITS_MAGIC ) - 1,
	};
	struct _im {
		u32 m_Index : MAX_BITS_INDEX;  // \[XzCfbNX
		u32 m_Magic : MAX_BITS_MAGIC;  // }WbNio[
	};
	union {
		_im m_IM;
		u32 m_unHandle;
	};
	
public:
	// LifeTime
	Handle() : m_unHandle(0){}
	void Init( u32 index );

	// Query
	u32  GetIndex () const { return m_IM.m_Index; }
	u32  GetMagic () const { return m_IM.m_Magic; }
	u32  GetHandle() const { return m_unHandle; }
	bool IsNull()    const { return (!m_unHandle); }

	operator u32 ()  const { return m_unHandle; }
};

template <typename TAG>
void Handle <TAG> :: Init( u32 index )
{
	assert( IsNull() );
	assert( index <= MAX_INDEX );

	static u32 s_AutoMagic = 0;
	if( ++s_AutoMagic > MAX_MAGIC ) {
		s_AutoMagic = 1;
	}

	m_IM.m_Index = index;
	m_IM.m_Magic = s_AutoMagic;
}

template <typename TAG>
inline bool operator != ( Handle <TAG> lhs, Handle <TAG> rhs ) {
	return ( lhs.GetHandle() != rhs.GetHandle() );
}

template <typename TAG>
inline bool operator == ( Handle <TAG> lhs, Handle <TAG> rhs ) {
	return ( lhs.GetHandle() == rhs.GetHandle() );
}

#endif //_HANDLE_H_
