/** @file handle_manager.h
 *  @brief HandleManagerNXwb_
 *
 *  @author yasufuku
 *  @date 2004/11
 *  @version 0.10
 *
 *  Copyright(C) 2004 Kensuke Yasufuku
 */
#ifndef _HANDLE_MANAGER_H_
#define _HANDLE_MANAGER_H_

#include "include.h"
#include <vector>

/**
 *	  @brief HandleNX
 *
 *    HandleNX
 *		GAME PROGRAMMING GEMSQlɂ܂
 */
template < typename DATA, typename HANDLE >
class HandleManager
{
private:
	typedef std::vector <DATA>	UserVec;
	typedef std::vector <u32>	MagicVec;
	typedef std::vector <u32>	FreeVec;

	UserVec		m_UserData;
	MagicVec	m_MagicNumbers;
	FreeVec		m_FreeSlots;

public:
	HandleManager(){ m_UserData.reserve(512);}
	~HandleManager(){}

	DATA * Acquire( HANDLE & handle );
	void   Release( HANDLE & handle );

	DATA *       Dereference( HANDLE handle );
	const DATA * Dereference( HANDLE handle ) const;

	u32  GetUsedHandleCount() const {
		return ( m_MagicNumbers.size() - m_FreeSlots.size() );
	}
	bool HasUsedHandles() const {
		return ( !! GetUsedHandleCount() );
	}
};

template < typename DATA, typename HANDLE >
DATA * HandleManager <DATA, HANDLE>::Acquire( HANDLE & handle )
{
	u32 index;
	
    if ( m_FreeSlots.empty() ) {
		
        index = (u32)m_MagicNumbers.size();
        handle.Init( index );
        m_UserData.push_back( DATA() );
        m_MagicNumbers.push_back( handle.GetMagic() );
		
    } else {
		
        index = m_FreeSlots.back();
        handle.Init( index );
        m_FreeSlots.pop_back();
        m_MagicNumbers[ index ] = handle.GetMagic();
    }
	return &*( m_UserData.begin() + index );
    //return ( &m_UserData[index] );
}

template <typename DATA, typename HANDLE>
void HandleManager <DATA, HANDLE>::Release( HANDLE & handle )
{
    // which one?
    u32 index = handle.GetIndex();

    // make sure it's valid
    assert( index < m_UserData.size() );
    assert( m_MagicNumbers[index] == handle.GetMagic() );

    // ok remove it - tag as unused and add to free list
    m_MagicNumbers[index] = 0;
    m_FreeSlots.push_back( index );
}

template <typename DATA, typename HANDLE>
inline DATA* HandleManager <DATA, HANDLE>
::Dereference( HANDLE handle )
{
    if( handle.IsNull() ) return (0);

    // check handle validity - $ this check can be removed for speed
    // if you can assume all handle references are always valid.
    u32 index = handle.GetIndex();

	if( ( index >= m_UserData.size() )
    ||  ( m_MagicNumbers[index] != handle.GetMagic() ) ) {
        // no good! invalid handle == client programming error
        assert(0);
        return (0);
    }
	return &*( m_UserData.begin() + index );
    //return ( &m_UserData[index] );
}

template <typename DATA, typename HANDLE>
inline const DATA* HandleManager <DATA, HANDLE>
:: Dereference( HANDLE handle ) const
{
    // this lazy cast is ok - non-const version does not modify anything
    typedef HandleManager <DATA, HANDLE> ThisType;
    return ( const_cast <ThisType*> (this)->Dereference( handle ) );
}
#endif //_HANDLE_H_
