/** @file task.cpp
 *  @brief TaskNXCvg
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.20
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */
//====================================================================
// CN[h
//====================================================================
#include "task.h"

//====================================================================
// 萔
//====================================================================
#define DEFAULT_TASK_NAME		"Anonym"; //ftHg() 

TaskMessage g_TMInit( TM_INIT );
TaskMessage g_TMMain( TM_MAIN, TM_TO_TREE );
TaskMessage g_TMKill( TM_KILL );
TaskMessage g_TMUpdate( TM_UPDATE );
TaskMessage g_TMRender( TM_RENDER, TM_TO_TREE );
TaskMessage g_TMResize( TM_RESIZE, TM_TO_TREE );

//====================================================================
// ^XNNX
//====================================================================
//--------------------------------------------------------------
// ֐
//--------------------------------------------------------------
/**
 *	@param pParent		e^XN
 *	@param nPriority	Dx
 */
Task * Task::Create( Task * pParent, int nPriority )
{
	Task * pTask = NEW Task;
	return pTask->Init( pParent, nPriority );
}

//--------------------------------------------------------------
// ֐
//--------------------------------------------------------------
/**
 *	@param pParent		e^XN
 *	@param nPriority	Dx
 *
 *  CreateɌĂяoDe^XNɐڑď
 */
Task * Task::Init( Task * pParent, int nPriority )
{
	if( pParent ) {
		pParent->AddChild( this, nPriority ); // q^XNɒǉ
	}
	
	if( Receive( &g_TMInit ) == TR_ERROR ){ // bZ[W
		Destroy();
		return (Task *)NULL;
	}

	m_bAlive = TRUE;
	
	return (Task *)this;
}
//--------------------------------------------------------------
// j֐
//--------------------------------------------------------------
/**
 *	q^XNCgdeleteD
 */
int Task::Destroy()
{
	Call( &g_TMKill ); // bZ[W
	
	// q^XN
	DestroyAllChildren();

	if( m_pParent ) {
		m_pParent->RemoveChild(this);
	}
	
	// g
	m_bAlive = FALSE;
	delete this;
	
	return TR_OK;
}

//--------------------------------------------------------------
// q^XNSĔj
//--------------------------------------------------------------
int Task::DestroyAllChildren()
{
	Task * pChild;
	if( IsAlive() ) {
		if( m_ChildList.size() > 0 ) {
			TaskList::iterator it = m_ChildList.begin();
	
			while( it != m_ChildList.end() ) {

				pChild = *it;
				it = m_ChildList.erase(it);
				pChild->Destroy();
			}
		}
	} else {
		PRINTF("%s:%d: warning: Task(%s)͂ł\n", __FILE__, __LINE__, m_pszName );
		return TR_ERROR;
	}
	return TR_OK;
}

//--------------------------------------------------------------
// RXgN^
//--------------------------------------------------------------
Task::Task()
{
	m_pszName   = DEFAULT_TASK_NAME;
	m_bAlive    = FALSE;
	m_nPriority = 0;
	m_nLock     = 0;
	m_pParent   = NULL;

}

//--------------------------------------------------------------
// fXgN^
//--------------------------------------------------------------
Task::~Task()
{
}

//--------------------------------------------------------------
// q^XNɒǉ
//--------------------------------------------------------------
int Task::AddChild( Task * pTask, int nPriority )
{
	pTask->SetPriority( nPriority );
	
	TaskList::iterator it = m_ChildList.begin();
	
	while( it != m_ChildList.end() ) {
		if( (*it)->GetPriority() > nPriority ) break;
		it++;
	}
	m_ChildList.insert( it, pTask );
	
	pTask->m_pParent = this; // eo^

	return TR_OK;
}

//--------------------------------------------------------------
// q^XN
//--------------------------------------------------------------
int Task::RemoveChild( Task * pTask )
{
	TaskList::iterator it = m_ChildList.begin();
	
	while( it != m_ChildList.end() ) {

		if( (*it) == pTask ) {
			m_ChildList.erase(it);
			return 0;
		}
		it++;
	}
	return 1;
}

//--------------------------------------------------------------
// bZ[WfBXpb`
//--------------------------------------------------------------
int Task::Receive( TaskMessage * pMessage )
{
	int nResult;

	switch( pMessage->Tag() )
	{
	  case TM_INIT:
		nResult = OnInit();
		break;
		
	  case TM_MAIN:
		nResult = OnMain();
		break;

	  case TM_KILL:
		nResult = OnKill();
		break;

	  default:
		break;
	}
	return nResult;
}

//--------------------------------------------------------------
// ^XN
//--------------------------------------------------------------
int Task::OnInit()
{
	return TR_OK;
}

//--------------------------------------------------------------
// C
//--------------------------------------------------------------
int Task::OnMain()
{
	return TR_OK;
}

//--------------------------------------------------------------
// n
//--------------------------------------------------------------
int Task::OnKill()
{
	return TR_OK;
}

//--------------------------------------------------------------
// TaskbZ[WM
//--------------------------------------------------------------
int Task::Send( TaskMessage * pMessage )
{
	return Receive( pMessage );
}

//--------------------------------------------------------------
// TaskbZ[W
//--------------------------------------------------------------
int Task::Call( TaskMessage * pMessage )
{
	int nResult = TR_OK;
	
	if( ! m_nLock ) {
		
		nResult = Send( pMessage );
		
		if( nResult == TR_OK && pMessage->Flag() == TM_TO_TREE ) {
			
			// ^XNs͎q^XNɂ
			TaskList::iterator it = m_ChildList.begin();
			while( it != m_ChildList.end() ) {

				nResult = (*it)->Call( pMessage );
				it++;
			}
		}
		
	} else {
		nResult = TR_LOCKED;
	}

	return nResult;
}

//--------------------------------------------------------------
// TaskbZ[W
//--------------------------------------------------------------
int Task::Call( TaskMessage Message )
{
	return Call( &Message );
}

//--------------------------------------------------------------
// OZbg
//--------------------------------------------------------------
int Task::SetName( char * pszName )
{
	m_pszName = pszName;
	return TR_OK;
}

//--------------------------------------------------------------
// vCIeBZbg
//--------------------------------------------------------------
void Task::SetPriority( int nPriority )
{
	m_nPriority = nPriority;
}

//--------------------------------------------------------------
// vCIeB擾
//--------------------------------------------------------------
int Task::GetPriority()
{
	return m_nPriority;
}

//--------------------------------------------------------------
// Lock
//--------------------------------------------------------------
int Task::Lock()
{
	m_nLock++;
	return 0;
}

//--------------------------------------------------------------
// Lock
//--------------------------------------------------------------
int Task::Unlock()
{
	if( m_nLock > 0 ) {
		m_nLock--;
	} else {
		PRINTF( "%s:%d: warning: Task(%s) is not Locked.\n", __FILE__, __LINE__, m_pszName );
	}
	return 0;
}

//--------------------------------------------------------------
// ̊mF
//--------------------------------------------------------------
int Task::IsAlive()
{
	return m_bAlive;
}

//--------------------------------------------------------------
// q^XN\
//--------------------------------------------------------------
void Task::DispChildren( int depth )
{	
	// 
	for( int i=0; i<depth; i++ ) {
		PRINTF("    ");
	}
	
	if( depth > 0 ) {
		PRINTF("+--- %s(%d)\n", m_pszName, m_nPriority );
	} else {
		PRINTF("%s(%d)\n", m_pszName, m_nPriority );
	}

	depth++;
	
	TaskList::iterator it = m_ChildList.begin();
	
	while( it != m_ChildList.end() ) {
		(*it)->DispChildren( depth );
		it++;
	}
}

//--------------------------------------------------------------
// q^XN擾
//--------------------------------------------------------------
Task * Task::Child( uint index )
{
	uint i = 0;
	TaskList::iterator it = m_ChildList.begin();
	
	while( it != m_ChildList.end() ) {
		if( i++ == index ) return (*it);
		it++;
	}
	return NULL;
}

//--------------------------------------------------------------
// e^XN̕ύX
//--------------------------------------------------------------
void Task::ChangeParent( Task * pParent )
{
	if( m_pParent ) {
		m_pParent->RemoveChild(this);
	}
	pParent->AddChild( this, m_nPriority );
}

//====================================================================
// [g^XNNX
//====================================================================
RootTask * RootTask::s_pSelf = NULL;

//--------------------------------------------------------------
// 
//--------------------------------------------------------------
RootTask * RootTask::Create()
{
	if( s_pSelf == NULL ) {
		s_pSelf = NEW RootTask;
		s_pSelf = (RootTask *)s_pSelf->Init( NULL, 0 );
	}
	return s_pSelf;
}

//--------------------------------------------------------------
// RXgN^
//--------------------------------------------------------------
RootTask::RootTask()
{
	SetName("RootTask");
}

//--------------------------------------------------------------
// fXgN^
//--------------------------------------------------------------
RootTask::~RootTask()
{
	DispTaskList();
}

//--------------------------------------------------------------
// C
//--------------------------------------------------------------
int RootTask::OnInit()
{
	SetName("RootTask");
	return TR_OK;
}

//--------------------------------------------------------------
// C
//--------------------------------------------------------------
int RootTask::OnMain()
{
	//PRINTF("RootTask OnMain\n");
	return TR_OK;
}

//--------------------------------------------------------------
// ^XNXg\
//--------------------------------------------------------------
void RootTask::DispTaskList()
{
	PRINTF("--------------------------------\n");
	PRINTF("  Task Tree\n");
	PRINTF("--------------------------------\n");

	DispChildren();

	PRINTF("--------------------------------\n");
}
