/** @file task.h
 *  @brief TaskNXwb_
 *
 *  @author yasufuku
 *  @date 2004/07
 *  @version 0.10
 *
 *  Copyright(C) 2004 Kensuke Yasufuku
 */
#ifndef _TASK_H_
#define _TASK_H_

//====================================================================
// CN[h
//====================================================================
#include <vector>
#include <list>
#include "task_message.h"

//====================================================================
// 񋓌^
//====================================================================
enum eTaskResult {
	TR_OK = 0,
	TR_ERROR,
	TR_LOCKED,
};

enum {
	TP_PROJECTIVE_MAP = 0,
	TP_SCENE,
	TP_CAMERA,
	TP_LIGHT,
	TP_MODEL,	
	TP_TRANS_MODEL,
	TP_FONT,
	TP_FADE,

	TP_VIEW
};

//====================================================================
// NX`
//====================================================================
/**
 *  @brief TaskNX
 *
 *	^XNVXeNX
*/
class Task
{
public:
	typedef std::list<Task *> TaskList; // ^XNXg(STLgp)

	//==============================================
	// oϐ
	//==============================================
private:
	char *   m_pszName;			///< ^XN
	bool     m_bAlive;			///< tO
	int      m_nPriority;		///< ^XNsDx

protected:
	int      m_nLock;			///< ^XNbN
	Task *   m_pParent;			///< e^XN
	TaskList m_ChildList;		///< q^XNXg

	//==============================================
	// o֐
	//==============================================
protected:
	//----------------------------------------------
	// bZ[W
	//----------------------------------------------
	virtual int Receive( TaskMessage * pMessage );	///< bZ[WfBXpb`
	virtual int OnInit();							///< ^XN
	virtual int OnMain();							///< ^XNC
	virtual int OnKill();							///< ^XN
	virtual int Send( TaskMessage * pMessage );		///< bZ[WM

public:
	//----------------------------------------------
	//  & j
	//----------------------------------------------
	/// ֐
	static Task * Create( Task * pParent, int nPriority );
	//template<class T> static T*  Create( Task * pParent, int nPriority );
	
	Task * Init( Task *parent, int nPri );	///< 
	int  Destroy();						///< j
	int  DestroyAllChildren();			///< q^XN̔j
	
	Task();								///< RXgN^
	virtual ~Task();					///< fXgN^

	//----------------------------------------------
	// eR}h
	//----------------------------------------------
	int  Call( TaskMessage * pMessage );///< ^XNĂяo(|C^)
	int  Call( TaskMessage   Message );	///< ^XNĂяo
	int  SetName( char * pszName );		///< Oݒ
	void SetPriority( int nPriority );	///< Dxݒ
	int  GetPriority();					///< Dx擾

	int  Lock();						///< Lock
	int  Unlock();						///< Lock
	int  IsAlive();						///< ̊mF
	int  AddChild( Task * pTask, int nPriority );	///< q^XNǉ
	int  RemoveChild( Task * pTask );
	char * Name() { return m_pszName; }	///< ^XN
	Task * Child( uint index );
	void ChangeParent( Task * pParent );
	Task * GetParent(){ return m_pParent; }

	//----------------------------------------------
	// fobOp
	//----------------------------------------------
	void DispChildren( int depth = 0 );		///< q^XN\
	
};

// O[oŎgpł^XNbZ[W
extern TaskMessage g_TMInit;
extern TaskMessage g_TMMain;
extern TaskMessage g_TMKill;
extern TaskMessage g_TMUpdate;
extern TaskMessage g_TMRender;
extern TaskMessage g_TMResize;

/**
 *  @brief RootTaskNX
 *
 *	^XÑ[gƂȂNX
*/
class RootTask : public Task
{
private:
	static RootTask * s_pSelf;	///< VOg
	RootTask();					///< RXgN^
	~RootTask();				///< fXgN^
protected:
	virtual int OnInit();		///< ^XN
	virtual int OnMain();		///< ^XNC
public:
	static RootTask * Create();	///< ֐
	void   DispTaskList();		///< ^XNXg̕\
	
};

#endif //_TASK_H_
