/** @file camera.cpp
 *  @brief CameraNX\[X
 *
 *  @author yasufuku
 *  @date 2004/07
 *  @version 0.10
 *
 *  Copyright(C) 2004 Kensuke Yasufuku
 */
#include "camera.h"
#include "scene.h"
#include "mathlib.h"
#include "sys_config.h"

extern SystemConfig sc;

//====================================================================
// CameraNXo֐`
//====================================================================
//---------------------------------------------------------------
// ֐
//---------------------------------------------------------------
/**
 * @param pParent		eGg
 * @param nPriority		^XNDx
 * @retval				ꂽCameraNX|C^
 */
Camera * Camera::Create( Element * pParent, int nPriority )
{
	Camera * pCam = NEW Camera;
	pCam->t.x = 0.0f;
	pCam->t.y = 0.0f;
	pCam->t.z = 1000.0f;
	pCam->lookat.x = 0.0f;
	pCam->lookat.y = 0.0f;
	pCam->lookat.z = 0.0f;
	pCam->m_nType = ORTHO;
	return (Camera *)pCam->Init( pParent, nPriority );
}

Camera * Camera::Create( Element * pParent, const Vector & pos, const Vector & at, int nPriority )
{
	Camera * pCam = NEW Camera;
	pCam->t       = pos;
	pCam->lookat  = at;
	pCam->m_nType = PERSPECTIVE;
	return (Camera *)pCam->Init( pParent, nPriority );
}

//---------------------------------------------------------------
// RXgN^
//---------------------------------------------------------------
Camera::Camera()
{
	SetName("Camera");
	lookat.Set();
	up.Set( 0.0f, 1.0f, 0.0f );
	m_fNear = 0.03f;
	m_fFar  = 1000.0f;
	m_fFovy = 34.516;
}

//----------------------------------------------------------------
// fXgN^
//----------------------------------------------------------------
Camera::~Camera()
{
}

//---------------------------------------------------------------
// XV
//---------------------------------------------------------------
int Camera::OnUpdate()
{
	UpdateMatrix();
	OnUpdateChildren();
	return TR_OK;
}

//---------------------------------------------------------------
// }gbNXXV
//---------------------------------------------------------------
int Camera::UpdateMatrix()
{
	// r[sXV
	m.Lookat( t, lookat, up );

	// ˉesXV
	if( m_nType == PERSPECTIVE ) {
		// e
		float aspect =  float(sc.nScreenWidth) / float(sc.nScreenHeight);
		proj.Perspective( m_fFovy, aspect, m_fNear, m_fFar );
	} else {
		// se
		float w = float(sc.nScreenWidth)*0.5f/s.x;
		float h = float(sc.nScreenHeight)*0.5f/s.y;
		proj.Ortho( -w, w, -h, h, m_fNear, m_fFar );
	}

	return 0;

}

//---------------------------------------------------------------
// _O
//---------------------------------------------------------------
int Camera::OnRender()
{
	glViewport( 0, 0, sc.nScreenWidth, sc.nScreenHeight );

	// VF[_[ΉɂJgV[Ƀ}gbNXZbg邾
	Scene::SetCameraPtr( this );
	
#if 0
	glDisable( GL_LIGHTING );
	glDisable( GL_LIGHT0 );
	glDisable( GL_TEXTURE_2D );
		
	// W\
	glBegin( GL_LINES );
		glColor3f(1.0, 1.0, 0.0);  
		glVertex3f(   0.0f, 0.0f, 0.0f );
		glVertex3f(100.0f, 0.0f, 0.0f );

		glColor3f(0.0, 1.0, 1.0);  
		glVertex3f(   0.0f, 0.0f, 0.0f );
		glVertex3f(   0.0f, 100.0f, 0.0f );

		glColor3f(1.0, 0.0, 1.0);  
		glVertex3f( 0.0f, 0.0f, 0.0f );
		glVertex3f( 0.0f, 0.0f, 100.0f );
	glEnd();
#endif

	// JOftHgݒ
	glEnable(GL_CULL_FACE); 
	glCullFace(GL_FRONT);

	return 0;
}
