/** @file material.cpp
 *  @brief MaterialNX
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.20
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */
#include "include.h"
#include "material.h"

//====================================================================
// MaterialNXo֐`
//====================================================================
//--------------------------------------------------------------
// 
//--------------------------------------------------------------
Material * Material::Create( MaterialData * pData,
							 eShaderMeshType shaderMeshType )
{
	Material * pMat = NEW Material;
	return pMat->Init( pData, shaderMeshType );
}

//--------------------------------------------------------------
// RXgN^
//--------------------------------------------------------------
Material::Material()
{
	m_pData = NULL;
	
	m_Specular.r = 1.0f;
	m_Specular.g = 1.0f;
	m_Specular.b = 1.0f;
	m_Specular.a = 1.0f;
	
	m_Diffuse.r = 1.0f;
	m_Diffuse.g = 1.0f;
	m_Diffuse.b = 1.0f;
	m_Diffuse.a = 1.0f;
	
	m_Ambient.r = 0.3f;
	m_Ambient.g = 0.3f;
	m_Ambient.b = 0.3f;
	m_Ambient.a = 1.0f;

	m_fShininess = 48; //K

	m_bValidTex = true;
}

//--------------------------------------------------------------
// fXgN^
//--------------------------------------------------------------
Material::~Material()
{
	TextureManager::Get().DeleteTexture(m_hTexture);
}

//--------------------------------------------------------------
// 
//--------------------------------------------------------------
Material * Material::Init( MaterialData * pData,
						   eShaderMeshType shaderMeshType )
{
	m_pData = pData;

	m_Specular = pData->specular;
	m_Diffuse  = pData->diffuse;
	m_Ambient  = pData->ambient;

	if( pData->sTexture > 0 ) {	
		Texture::HeaderTbl ht = { pData->sTexture, pData->pTexHeader };
		m_hTexture = TextureManager::Get().GetHandle( ht );
	}

	switch( shaderMeshType )
	{
		case STATIC_MESH_SHADER:
			if( pData->sTexture > 1 ) {
				m_pShader = ShaderManager::Get()->StaticDiffuseMapLightMap();
			} else if( pData->sTexture == 1 ) {
				if( pData->bNoLight )  {
					m_pShader = ShaderManager::Get()->StaticDiffuseMap();
				} else {
					m_pShader = ShaderManager::Get()->StaticLambertDiffuseMap();
				}
			} else {
				//m_pShader = ShaderManager::Get()->StaticLambert();
				m_pShader = ShaderManager::Get()->StaticPhong();
			}
			break;
		case SKIN_MESH_SHADER:
			if( pData->sTexture > 0 ) {

			} else {
				m_pShader = ShaderManager::Get()->SkinLambert();
				//m_pShader = ShaderManager::Get()->StaticPhong();
			}
			break;
	}
	
	return this;
}

//--------------------------------------------------------------
// Fill
//--------------------------------------------------------------
void Material::Fill()
{
	m_pShader->Fill();

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glUniform4fv( m_pShader->LocDiffuse(), 1, (float *)&m_Diffuse.r );
	glUniform4fv( m_pShader->LocAmbient(), 1, (float *)&m_Ambient.r );
		
	// eNX`FILL
	Texture * pTex = TextureManager::Get().GetTexturePtr(m_hTexture);
	if( pTex && m_bValidTex ) {
		pTex->Fill( m_pShader->LocDiffuseMap(), m_pShader->LocLightMap() );
	} else {	
		glDisable( GL_TEXTURE_2D );
	}
}

//--------------------------------------------------------------
// ǂ`FbN
//--------------------------------------------------------------
bool Material::CheckTrans()
{
	if( m_Diffuse.a < 1.0f ) return true;
	return false;
}

//--------------------------------------------------------------
// eNX`TCY擾
//--------------------------------------------------------------
u32 Material::GetTexSize()
{
	Texture * p = TextureManager::Get().GetTexturePtr(m_hTexture);
	if( ! p ) return 0;
	return p->GetSize();
}

u32 Material::GetTexNum()
{
	Texture * p = TextureManager::Get().GetTexturePtr(m_hTexture);
	if( ! p ) return 0;
	return p->GetNum();
}
