/** @file model.cpp
 *  @brief 3D ModelNXCvg
 *
 *  @author yasufuku
 *  @date 2011/01
 *  @version 0.10
 *
 *  Copyright(C) 2011 Kensuke Yasufuku
 */
#include "model.h"
#include "scene.h"
#include "skin_mesh.h"
#include "mathlib.h"
#include "shader_manager.h"

//====================================================================
// ModelNXo֐`
//====================================================================
//---------------------------------------------------------------
// 
//---------------------------------------------------------------
/**
 *	@param pParent		eGg
 *	@param pData		ff[^
 *	@param nPriority	Dx
 *	@retval				ꂽModelNX|C^
 */
Model * Model::Create( Element * pParent, ModelData * pData, int nPriority )
{
	Model * pMdl = NEW Model;
	pMdl->m_pData = pData;
	return (Model *)pMdl->Init( pParent, nPriority );
}

//---------------------------------------------------------------
// RXgN^
//---------------------------------------------------------------
Model::Model()
{
	SetName("Model");

	// [NA
	m_pData     = NULL;
	m_pMesh     = NULL;
	m_pMotion     = NULL;
}

//---------------------------------------------------------------
// fXgN^
//---------------------------------------------------------------
Model::~Model()
{
	m_pMesh->Release();
	if( m_pMotion     ) delete m_pMotion;
}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
int Model::OnInit()
{
	// bV
	m_pMesh = (MeshBase *)SkinMesh::Create( m_pData->pMesh, m_pData->sBone, m_pData->sWeights );
	m_pMesh->SetVtxFmt( (int)m_pData->unFlag );


	return 0;
}

//---------------------------------------------------------------
// XV
//---------------------------------------------------------------
int Model::OnUpdate()
{	
	// }gbNXXV
	Element::OnUpdate();

	// [VXV
	if( m_pMotion ) {
		m_pMotion->UpdateMatrixList( m_pData );
		m_pMotion->UpdateWeightMatrixList( m_pData );
	}	
	return 0;
}

//---------------------------------------------------------------
// _O
//---------------------------------------------------------------
int Model::OnRender()
{
	// VF[_[ΉɂJgV[Ƀ}gbNXZbg邾
	Scene::SetModelMatrixPtr( &m );

	// FILL
	if( m_pMotion ) m_pMesh->FillMotion( m_pMotion->GetMatrixList(), m_pData->sBone + m_pData->sWeights );
	m_pMesh->Fill();

	return 0;
}

//---------------------------------------------------------------
// [Vݒ
//---------------------------------------------------------------
void Model::SetMotion(  MotionData * pMotion, float fBlend )
{
	if( m_pMotion == NULL ) {
		m_pMotion = NEW Motion;
	}
	m_pMotion->SetData( pMotion, fBlend );
	m_pMotion->UpdateMatrixList( m_pData );
	m_pMotion->UpdateWeightMatrixList( m_pData );
}

//---------------------------------------------------------------
// [V̎Ԃ߂
//---------------------------------------------------------------
void Model::IncTime( float fTime )
{
	if( m_pMotion ) {
		m_pMotion->IncTime( fTime );
	}
}
