/** @file point_sprite.cpp
 *  @brief Point SpriteNXCvg
 *
 *  @author yasufuku
 *  @date 2011/03
 *  @version 0.20
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */
#include "point_sprite.h"
#include "matrix.h"
#include "scene.h"
#include "shader_manager.h"

//===============================================================
// Point SpriteNXo֐`
//===============================================================
//---------------------------------------------------------------
// RXgN^
//---------------------------------------------------------------
PointSprite::PointSprite()
{
	// [NA
	m_unVSize = 0;
	m_unCount = 0;
	m_VBO = 0;
	m_pShader = NULL;
}

//---------------------------------------------------------------
// fXgN^
//---------------------------------------------------------------
PointSprite::~PointSprite()
{
	glDeleteBuffersARB( 1, &m_VBO );
}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
int PointSprite::Init( size_t size, u32 count, float * pData )
{
	m_unVSize = (u32)size;
	m_unCount = count;
	
	// VF[_[ݒ
	m_pShader = ShaderManager::Get()->PointSprite();
	
	// VBÕj[NID
	glGenBuffers( 1, &m_VBO );
	// VBOBind
	glBindBuffer( GL_ARRAY_BUFFER, m_VBO );
	// VBÕm
#if _ENABLE_CUDA_
	glBufferData( GL_ARRAY_BUFFER, size*count, 0, GL_DYNAMIC_DRAW );
#else
	glBufferData( GL_ARRAY_BUFFER, size*count, pData, GL_STATIC_DRAW );
#endif

	glVertexAttribPointer( m_pShader->LocPosition(), 4, GL_FLOAT, false,
						   sizeof(GL_FLOAT)*4,
						   0 );
	glEnableVertexAttribArray( m_pShader->LocPosition() );

	
	// VBOBind
	glBindBuffer( GL_ARRAY_BUFFER, 0 );

	return 0;
}

//---------------------------------------------------------------
// `
//---------------------------------------------------------------
int PointSprite::Draw( float * pData )
{
	float psize = 8.0f;

	static const Matrix m;
	Scene::SetModelMatrixPtr( &m );

	const Matrix * pMVP = Scene::GetCurrentModelViewProjMatrixPtr();
	const Camera * pCam = Scene::GetCurrentCameraPtr();

	// VF[_[Lɂ
	m_pShader->Fill();
	
	glEnableVertexAttribArray( m_pShader->LocPosition() );
	glUniformMatrix4fv( m_pShader->LocMVPMatrix(), 1, GL_FALSE, (float *)pMVP->rc );
	glBindBuffer( GL_ARRAY_BUFFER, m_VBO );

	// VBÕf[^`
	{
		glDisable(GL_POINT_DISTANCE_ATTENUATION);
		glDisable(GL_POINT_SMOOTH);
		glDisable(GL_LIGHTING);
		glDisable(GL_BLEND);
		glDisable( GL_TEXTURE_2D );
	}

	{
		glColor3f( 0.8f, 0.0f, 0.0f );
		glPointSize(pCam->s.x*0.6f);
		
		//glVertexPointer( 4, GL_FLOAT, 0, 0 );
	}
	if( pData ) glBufferData( GL_ARRAY_BUFFER, m_unVSize*m_unCount, pData, GL_STATIC_DRAW );
	glDrawArrays(GL_POINTS, 0, m_unCount );

	
	glBindBuffer( GL_ARRAY_BUFFER, 0 );

	return 0;
}
