/** @file primitive.h
 *  @brief PrimitiveNXwb_
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.20
 *
 *  Copyright(C) 2011 Kensuke Yasufuku
 */
#ifndef _PRIMITIVE_H_
#define _PRIMITIVE_H_

#include "include.h"
#include "element.h"
#include "color.h"
#include "model_data.h"
#include "shader_base.h"

namespace pge {

/**
 *  @brief PrimitiveNX
 *
 *	v~eBuNX
 */
class Primitive : public Element
{
protected:
	ShaderBase * m_pShader;
	GLuint     * m_paVAO;
	GLuint     * m_paVBO;
	Color    m_Color;

	Primitive(){ m_pShader=NULL; m_paVAO=NULL; m_paVBO=NULL; }
	virtual ~Primitive(){;}
	virtual int OnUpdate(){ return Element::OnUpdate(); }
};
	
/**
 *  @brief LineNX
 *
 *	NX
 */
class Line : public Primitive
{
private:
	GLfloat  m_Vertices[6];

	Line();
	virtual ~Line();
	
protected:
	virtual int OnInit();	
	virtual int OnRender();
	
public:
	static Line * Create( Element * pParent,
						   const Vector & s,
						   const Vector & e,
						   const Color  & c );
	void SetColor( const Color & c );
	void SetPoints( const Vector & s, const Vector & e );
};

/**
 *  @brief ArrowNX
 *
 *	3DNX
 */
class Arrow : public Primitive
{
private:
	float		 m_fLength;
	Vector       m_vStart;

	Matrix       m_mLookat;
	GLuint     * m_paEBO;

	Arrow();
	virtual ~Arrow();
protected:
	virtual int OnInit();
	virtual int OnRender();
	
public:
	static Arrow * Create( Element * pParent,
						   const Vector & s,
						   const Vector & e,
						   const Color  & c );
	void SetColor( const Color & c );
	void SetPoints( const Vector & s, const Vector & e );
};

};
#endif //_PRIMITIVE_H
