/*  @brief SpriteNXCvg
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.20
 *
 *  Copyright(C) 2007 Kensuke Yasufuku
 */
#include "sprite.h"
#include "shader_manager.h"
#include "scene.h"
#include "texture_manager.h"
#include "keyboard.h"

//===============================================================
// SpriteNXo֐
//===============================================================
//---------------------------------------------------------------
// 
//---------------------------------------------------------------
Sprite * Sprite::Create( Element * pParent, const char * tgafile, float w, float h )
{
	Sprite * pSprite = NEW Sprite(tgafile,w,h);
	pSprite->Init( pParent, TP_MODEL );
	return pSprite;
}

//---------------------------------------------------------------
// RXgN^
//---------------------------------------------------------------
Sprite::Sprite( const char * tgafile, float w, float h )
{
	SetName("Sprite");
	
	m_pShader  = NULL;
	m_paVAO    = NULL;
	m_paVBO    = NULL;

	m_TexImage.Set(tgafile);
	m_TexHeader.Set(&m_TexImage);
	
	Texture::HeaderTbl ht = { 1, &m_TexHeader };
	m_hTexture = TextureManager::Get().GetHandle( ht );

	// _obt@f[^쐬
	m_Vertices[0].x  = -w*0.5f; // pos x
	m_Vertices[0].y  = -h*0.5f; // pos y
	m_Vertices[0].z  = 0.0f; // pos z
	m_Vertices[0].r  = 1.0f; // r
	m_Vertices[0].g  = 1.0f; // g
	m_Vertices[0].b  = 1.0f; // b
	m_Vertices[0].a  = 1.0f; // a
	m_Vertices[0].u  = 0.0f; // u
	m_Vertices[0].v  = 0.0f; // v

	m_Vertices[1].x = -w*0.5f;    // pos x
	m_Vertices[1].y =  h*0.5f; // pos y
	m_Vertices[1].z = 0.0f; // pos z
	m_Vertices[1].r = 1.0f; // r
	m_Vertices[1].g = 1.0f; // g
	m_Vertices[1].b = 1.0f; // b
	m_Vertices[1].a = 1.0f; // a
	m_Vertices[1].u = 0.0f; // u
	m_Vertices[1].v = 1.0f; // v

	m_Vertices[2].x = w*0.5f;    // pos x
	m_Vertices[2].y = h*0.5f;    // pos y
	m_Vertices[2].z = 0.0f; // pos z
	m_Vertices[2].r = 1.0f; // r
	m_Vertices[2].g = 1.0f; // g
	m_Vertices[2].b = 1.0f; // b
	m_Vertices[2].a = 1.0f; // a
	m_Vertices[2].u = 1.0f; // u
	m_Vertices[2].v = 1.0f; // v

	m_Vertices[3].x =  w*0.5f; // pos x
	m_Vertices[3].y = -h*0.5f;    // pos y
	m_Vertices[3].z = 0.0f; // pos z
	m_Vertices[3].r = 1.0f; // r
	m_Vertices[3].g = 1.0f; // g
	m_Vertices[3].b = 1.0f; // b
	m_Vertices[3].a = 1.0f; // a
	m_Vertices[3].u = 1.0f; // u
	m_Vertices[3].v = 0.0f; // v

}

//---------------------------------------------------------------
// fXgN^
//---------------------------------------------------------------
Sprite::~Sprite()
{
}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
int Sprite::OnInit()
{
	m_pShader = ShaderManager::Get()->Sprite();

	m_paVAO = NEW GLuint[1];
	m_paVBO = NEW GLuint[1];
	
	glGenVertexArrays( 1, m_paVAO );
	glGenBuffers( 1, m_paVBO );

	glBindVertexArray( m_paVAO[0] );
	glBindBuffer( GL_ARRAY_BUFFER, m_paVBO[0] );
	glBufferData( GL_ARRAY_BUFFER,
				  sizeof(GLfloat)*SPRITE_VERTICES_SIZE,
				  m_Vertices,
				  GL_STATIC_DRAW );
	
	glVertexAttribPointer( m_pShader->LocPosition(), 3, GL_FLOAT, false,
						   sizeof(GL_FLOAT)*SPRITE_VERTICES_SIZE/4,
						   0 );
	glEnableVertexAttribArray( m_pShader->LocPosition() );

	glVertexAttribPointer( m_pShader->LocVColor(), 4, GL_FLOAT, false,
						   sizeof(GL_FLOAT)*SPRITE_VERTICES_SIZE/4,
						   reinterpret_cast<void*>(sizeof(float)*3) );
	glEnableVertexAttribArray( m_pShader->LocVColor() );

	glVertexAttribPointer( m_pShader->LocUV0(), 2, GL_FLOAT, false,
						   sizeof(GL_FLOAT)*SPRITE_VERTICES_SIZE/4,
						   reinterpret_cast<void*>(sizeof(float)*7) );
	glEnableVertexAttribArray( m_pShader->LocUV0() );
	
	glBindBuffer( GL_ARRAY_BUFFER, 0 );
	glBindVertexArray( 0 );
		
	return TR_OK;
}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
int Sprite::OnKill()
{
	glDeleteVertexArrays( 1, m_paVAO );
	glDeleteBuffers( 1, m_paVBO );
	delete [] m_paVAO; m_paVAO = NULL;
	delete [] m_paVBO; m_paVBO = NULL;

	TextureManager::Get().DeleteTexture(m_hTexture);

	return TR_OK;
}

//---------------------------------------------------------------
// _O
//---------------------------------------------------------------
int Sprite::OnRender()
{
	static const uint FONT_TEX_WIDTH    = 2048;
	static const uint FONT_TEX_HEIGHT   = 2048;
	static const uint FONT_TEX_LETTER_X = 256;
	static const uint FONT_TEX_LETTER_Y = 256;


	static char frame = 0;
	static int timer = 0;
	float u1, v1, u2, v2;

	Scene::SetModelMatrixPtr( &m );
	const Matrix * pMVP = Scene::GetCurrentModelViewProjMatrixPtr();

	m_pShader->Fill();
	Texture * p = TextureManager::Get().GetTexturePtr(m_hTexture);
	if( p ) p->Fill( m_pShader->LocDiffuseMap() );
	glBindVertexArray( m_paVAO[0] );
	glBindBuffer( GL_ARRAY_BUFFER, m_paVBO[0] );

	// 
	if(0){
		

		if( Keyboard::Press(KEY_RIGHT) ) {timer++;

		if( timer%2 == 0 ) {
			timer=0;
			frame++;
			frame %= 64;

			u1 = (float)((int)(frame%8) * FONT_TEX_LETTER_X );
			v1 = (float)((int)(frame/8) * FONT_TEX_LETTER_Y );

			u2 = u1 + FONT_TEX_LETTER_X;
			v2 = v1 + FONT_TEX_LETTER_Y;

			u1 = (u1+0.5f) / FONT_TEX_WIDTH;
			u2 = (u2+0.5f) / FONT_TEX_WIDTH;
			v1 = (v1+0.5f) / FONT_TEX_HEIGHT;
			v2 = (v2+0.5f) / FONT_TEX_HEIGHT;

			m_Vertices[0].u  = u1;
			m_Vertices[0].v  = 1.0f-v2;

			m_Vertices[1].u  = u2;
			m_Vertices[1].v  = 1.0f-v2;

			m_Vertices[2].u  = u2;
			m_Vertices[2].v  = 1.0f-v1;

			m_Vertices[3].u  = u1;
			m_Vertices[3].v  = 1.0f-v1;



			glBufferData( GL_ARRAY_BUFFER,
						  sizeof(GLfloat)*SPRITE_VERTICES_SIZE,
						  m_Vertices,
						  GL_STATIC_DRAW );
		}
	}
	}
	glDepthMask(GL_FALSE);
	glUniformMatrix4fv( m_pShader->LocMVPMatrix(), 1, GL_FALSE, (float *)pMVP->rc );

	glEnableVertexAttribArray( m_pShader->LocPosition() );
	glEnableVertexAttribArray( m_pShader->LocVColor()    );
	glEnableVertexAttribArray( m_pShader->LocUV0()      );
	glDrawArrays( GL_QUADS, 0, 4 );
	
	glDepthMask(GL_TRUE);
	glBindBuffer( GL_ARRAY_BUFFER, 0 );
	glBindVertexArray(0);


	return TR_OK;
};
