/** @file mathlib.h
 *  @brief wCuwb_
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.20
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */
#ifndef _MATHLIB_H_
#define _MATHLIB_H_

#include "include.h"
#include <stdlib.h>
#include <math.h>

class Vector;
//================================================================
// define
//================================================================
#ifndef PI
#define PI			(3.14159265358979323846F)
#endif

#ifndef PI2
#define PI2			(3.14159265358979323846F*2.0F)
#endif

#ifndef N2
#define N2( x )		((x)*(x))
#endif

#ifndef	SIGN
#define	SIGN(x)		( ( (x) < 0 ) ? -1 : ( (x) > 0 ) ? 1 : 0 )
#endif

#define FRACTIONAL_ZERO  0.0001
#define	DELTA        1.0e-6f

float	const	TOLERANCE = 0.000000000000001f;		// float type tolerance

//================================================================
// CC֐
//================================================================
//---------------------------------------------
// degree <--> radian
//---------------------------------------------
inline float DegToRad(float a) { return a*0.01745329252f;}
inline float RadToDeg(float a) { return a*57.29577951f;}
//---------------------------------------------
// [
//---------------------------------------------
inline bool  IsNearlyZero  (float  f) { return (fabsf(f) < (float)FRACTIONAL_ZERO); }
inline bool  IsNearlyZero  (double d) { return (fabs(d)  < FRACTIONAL_ZERO);    }

extern int RoundOff45( float f );
//---------------------------------------------
// 
//---------------------------------------------
inline int GetRandom(  int l, int r ){ return ( l + (int)( (float)(r-l)*rand()/(RAND_MAX+1.0) ) ); }
inline int GetRandom2( int l, int r ){ return ( l + (int)( (float)(r-l)*rand()*rand()/(RAND_MAX*RAND_MAX+1.0) ) ); }

// Random֐
inline int Random(int maximumValue) {
	return (int)(((long long)rand() * maximumValue) / (1 + RAND_MAX));
}
inline float Random(float maximumValue) {
	return (rand() * (maximumValue * (1.0f / RAND_MAX)));
}

//---------------------------------------------
// NXpȂ኱ȃ}gbNXZ
//---------------------------------------------
typedef struct _Mtx44 {
	float m[4][4];
} Mtx44;

typedef struct _TRS {
	float trans[3];
	float quat[4];
	float scale[3];
} TRS;

typedef struct _Vec3 {
	float x;
	float y;
	float z;
} Vec3;

typedef struct _Vec2 {
	float x;
	float y;
	float z;
} Vec2;

inline void MulMtxTRS( float * dst, float rc[4][4], float * src )
{

	dst[0] = src[0]*rc[0][0] + src[1]*rc[1][0] + src[2]*rc[2][0] + 1.0f*rc[3][0];
	dst[1] = src[0]*rc[0][1] + src[1]*rc[1][1] + src[2]*rc[2][1] + 1.0f*rc[3][1];
	dst[2] = src[0]*rc[0][2] + src[1]*rc[1][2] + src[2]*rc[2][2] + 1.0f*rc[3][2];

}
inline void MulMtxRS( float * dst, float rc[4][4], float * src )
{
	dst[0] = src[0]*rc[0][0] + src[1]*rc[1][0] + src[2]*rc[2][0];
	dst[1] = src[0]*rc[0][1] + src[1]*rc[1][1] + src[2]*rc[2][1];
	dst[2] = src[0]*rc[0][2] + src[1]*rc[1][2] + src[2]*rc[2][2];
}
inline void MtxMulScalar( float dst[4][4], float src[4][4], float scalar )
{
	float * d = &dst[0][0];
	float * s = &src[0][0];	
	for( int i=0; i<16; i++ ) {
		*d++ = *s++ * scalar;
	}
}
inline void PlusMtxMulScalar( float dst[4][4], float src[4][4], float scalar )
{
	float * d = &dst[0][0];
	float * s = &src[0][0];	
	for( int i=0; i<16; i++ ) {
		*d++ += *s++ * scalar;
	}
}
inline void MtxMulMtx( float dst[4][4], float src1[4][4], float src2[4][4] )
{
	int r, c;
	for( r=0; r<4; r++ ) {
		for( c=0; c<4; c++ ) {
			dst[c][r]  = src1[0][r] * src2[c][0];
			dst[c][r] += src1[1][r] * src2[c][1];
			dst[c][r] += src1[2][r] * src2[c][2];
			dst[c][r] += src1[3][r] * src2[c][3];
		}
	}
}

#define	_CopyVector4( /* Vector4f */ vToVector,				\
					  /* Vector4f */ vFromVector )			\
{															\
	u32		*__p0 = (u32 *)( vToVector );					\
	u32		*__p1 = (u32 *)( vFromVector );					\
	__p0[0] = __p1[0], __p0[1] = __p1[1],					\
	__p0[2] = __p1[2], __p0[3] = __p1[3];					\
}

#define	_LerpVector3( /* Vector4f */ c,						\
					  /* Vector4f */ a,						\
					  /* Vector4f */ b,						\
					  /* float    */ t )					\
{															\
	(c)[0] = ( (a)[0] - (b)[0] ) * (t) + (b)[0],			\
	(c)[1] = ( (a)[1] - (b)[1] ) * (t) + (b)[1],			\
	(c)[2] = ( (a)[2] - (b)[2] ) * (t) + (b)[2];			\
}

#define	_LerpVector4( /* Vector4f */ c,						\
					  /* Vector4f */ a,						\
					  /* Vector4f */ b,						\
					  /* float    */ t )					\
{															\
	(c)[0] = ( (a)[0] - (b)[0] ) * (t) + (b)[0],			\
	(c)[1] = ( (a)[1] - (b)[1] ) * (t) + (b)[1],			\
	(c)[2] = ( (a)[2] - (b)[2] ) * (t) + (b)[2],			\
	(c)[3] = ( (a)[3] - (b)[3] ) * (t) + (b)[3];			\
}

#define	_DotProduct4( /* Vector4f */ a,						\
					  /* Vector4f */ b )					\
(															\
	(a)[0] * (b)[0] + (a)[1] * (b)[1] +						\
	(a)[2] * (b)[2] + (a)[3] * (b)[3]						\
)

#define	_T2LerpVector4( /* Vector4f */ c,					\
					    /* Vector4f */ a,					\
					    /* Vector4f */ b,					\
					    /* float    */ t0,					\
					    /* float    */ t1 )					\
{															\
	(c)[0] = (a)[0] * (t0) + (b)[0] * (t1),					\
	(c)[1] = (a)[1] * (t0) + (b)[1] * (t1),					\
	(c)[2] = (a)[2] * (t0) + (b)[2] * (t1),					\
	(c)[3] = (a)[3] * (t0) + (b)[3] * (t1);					\
}

#define	_ScalarVector4( /* Vector4f */ c,					\
						/* Vector4f */ a,					\
						/* float    */ b )					\
{															\
	(c)[0] = (a)[0] * (b), (c)[1] = (a)[1] * (b),			\
	(c)[2] = (a)[2] * (b), (c)[3] = (a)[3] * (b);			\
}


//===========================================================
// [VԌn
//===========================================================
void CopyTRS( int nBone, TRS * pTrsDst, TRS * pTrsSrc );
void SlerpQuat( int   nBone,   TRS   * pDst,
				TRS   * pKeyA, TRS   * pKeyB, float t );
void LerpTrans( int   nBone,   TRS   * pDst,
				TRS   * pKeyA, TRS   * pKeyB, float t );
void LerpScale( int   nBone,   TRS   * pDst,
				TRS   * pKeyA, TRS   * pKeyB, float t );


#endif //_MATHLIB_H_
