/** @file matrix.h
 *  @brief MatrixNXwb_
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.06
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */
#ifndef _MATRIX_H_
#define _MATRIX_H_

#include "include.h"
#include "mathlib.h"
#include "vector.h"

/**
 *	  @brief MatrixNX
 *
 *    s񉉎ZNX ( s͂S~S float̂ݑΉ )
 */
class Matrix
{
public:
	
	float rc[4][4]; ///< (Row&Column)

	//	| rc[0][0] rc[0][1] rc[0][2] rc[0][3] |   | axisX[x] axisX[y] axisX[z] 0 |
	//	| rc[1][0] rc[1][1] rc[1][2] rc[1][3] | = | axisY[x] axisY[y] axisY[z] 0 |
	//	| rc[2][0] rc[2][1] rc[2][2] rc[2][3] |   | axisZ[x] axisZ[y] axisZ[z] 0 |
	//	| rc[3][0] rc[3][1] rc[3][2] rc[3][3] |   |     t[x]     t[y]     t[z] 1 |


public:
	Matrix();
	Matrix( const Matrix & m );
	Matrix( const float m[4][4] );
	virtual ~Matrix();

	void Set( const float m[4][4] );
	// Zq
	Matrix & operator =  ( const Matrix &m );
	Matrix & operator *= ( float scalar );
	Matrix & operator /= ( float scalar );
	Matrix & operator *= ( const Matrix &m );
	Matrix & operator += ( const Matrix &m );
	Matrix & operator -= ( const Matrix &m );
	
	Matrix   operator *  ( float scalar ) const;
	Matrix   operator /  ( float scalar ) const;
	Matrix   operator *  ( const Matrix &m ) const;
	Matrix   operator +  ( const Matrix &m ) const;
	Matrix   operator -  ( const Matrix &m ) const;

	Vector   operator *  ( const Vector &v ) const;

	// Operation
	Matrix & Unit();			///< Pʍs
	Matrix & Zero();			///< s
	Matrix & Transpose();		///< ]us
	Matrix & Invert();			///< ts

	/// ]us
	Matrix TransposedMatrix() const;
	/// ts
	Matrix InverseMatrix() const;

	// Translation
	Matrix & TransR( const Vector & t );
	Matrix & TransR( float x, float y, float z );
	Matrix & TransL( const Vector & trans );
	Matrix & TransL( float x, float y, float z );
	Matrix & SetTrans( const Vector & t );
	Matrix & SetTrans( float x, float y, float z );

	Vector GetTrans(){ return Vector( rc[3][0], rc[3][1], rc[3][2], 1.0f ); }
	
	// Rotation
	Matrix & RotX( float x );
	Matrix & RotY( float y );
	Matrix & RotZ( float z );
	Matrix & RotXYZ( const Vector & r );
	Matrix & RotXYZ( float x, float y, float z );
	Matrix & RotZYX( const Vector & r );
	Matrix & RotZYX( float x, float y, float z );

	// Scale
	Matrix & ScaleR( const Vector & s );
	Matrix & ScaleR( float x, float y, float z );
	Matrix & ScaleL( const Vector & s );
	Matrix & ScaleL( float x, float y, float z );

	// View
	Matrix & Lookat( const Vector & t, const Vector & at, const Vector & up );

	// Projection
	Matrix & Perspective( float fFovy, float fAspect, float fNear, float fFar );
	Matrix & Ortho( float left, float right, float bottom, float top, float near, float far );

	void Print();
};


#endif //_MATRIX_H_
