/** @file vector.cpp
 *  @brief VectorNXCvg
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.20
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */

//====================================================================
// CN[h
//====================================================================
#include "vector.h"

//====================================================================
// statico
//====================================================================


//====================================================================
// VectorNXo֐`
//====================================================================
//--------------------------------------------------------------------
// RXgN^
//--------------------------------------------------------------------
Vector::Vector()
:
x(0.0f),
y(0.0f),
z(0.0f),
w(1.0f)
{
	
}

Vector::Vector( float fx, float fy, float fz, float fw )
:
x(fx),
y(fy),
z(fz),
w(fw)
{
}

Vector::Vector( const Vector & v )
:
x(v.x),
y(v.y),
z(v.z),
w(v.w)
{
}

//--------------------------------------------------------------------
// fXgN^
//--------------------------------------------------------------------
Vector::~Vector()
{
}

//--------------------------------------------------------------------
// zCfbNX
//--------------------------------------------------------------------
float & Vector::operator [] ( u32 i )
{
	//assert( i<4 );
	return(i == 0) ? x : (i == 1) ? y : (i == 2) ? z : w;
}

const float	& Vector::operator [] ( u32 i ) const
{
	//assert( i<4 );
	return(i == 0) ? x : (i == 1) ? y : (i == 2) ? z : w;
}

//--------------------------------------------------------------------
// Ȃ̔
//--------------------------------------------------------------------
bool operator == ( const Vector &a, const Vector &b )
{
	return ( (a.x == b.x) && (a.y == b.y) && (a.z == b.z) && (a.w == b.w) );
}

bool operator != ( const Vector &a, const Vector &b )
{
	return( (a.x != b.x) || (a.y != b.y) || (a.z != b.z) || (a.w != b.w) );
}

//--------------------------------------------------------------------
// ]
//--------------------------------------------------------------------
Vector operator - ( const Vector &a )
{
	return Vector( -a.x, -a.y, -a.z );
}

//--------------------------------------------------------------------
// Z
//--------------------------------------------------------------------
Vector & Vector::operator += ( const Vector &v )
{
	x += v.x;
    y += v.y;
    z += v.z;

    return * this;
}

Vector & Vector::operator -= ( const Vector &v )
{
	x -= v.x;
    y -= v.y;
    z -= v.z;

    return * this;
}

Vector operator + ( const Vector &a, const Vector &b )
{
	Vector ret(a);
	ret += b;
	return ret;
}

Vector operator - ( const Vector &a, const Vector &b )
{
	Vector ret(a);
	ret -= b;
	return ret;
}

//--------------------------------------------------------------------
// XJ[{
//--------------------------------------------------------------------
Vector & Vector::operator *= ( float f )
{
	x *= f;
    y *= f;
    z *= f;

    return * this;
}

Vector & Vector::operator /= ( float f )
{
	x /= f;
    y /= f;
    z /= f;

    return * this;
}

Vector operator * ( const Vector &v, float f )
{
	return Vector( f*v.x, f*v.y, f*v.z );
}

Vector operator * ( float f, const Vector &v )
{
	return Vector( f*v.x, f*v.y, f*v.z );
}

Vector operator / ( const Vector &v, float f )
{
	return Vector( v.x/f, v.y/f, v.z/f );
}

//--------------------------------------------------------------------
// 
//--------------------------------------------------------------------
float operator * ( const Vector &a, const Vector &b )
{
	return a.x*b.x + a.y*b.y + a.z*b.z;
}

//-------------------------------------------------
// lZbg
//-------------------------------------------------
void Vector::Set( float fx, float fy, float fz, float fw )
{
	x = fx;
	y = fy;
	z = fz;
	w = fw;
}

//-------------------------------------------------
// 擾
//-------------------------------------------------
float Vector::Length() const
{
	return sqrtf( x*x + y*y + z*z );
}

//-------------------------------------------------
// ̂Q擾
//-------------------------------------------------
float Vector::LengthSqr() const
{
	return ( x*x + y*y + z*z );
}

//-------------------------------------------------
// 擾
//-------------------------------------------------
float Vector::LengthXZ() const
{
	return sqrtf( x*x + z*z );
}

//-------------------------------------------------
// [
//-------------------------------------------------
bool Vector::IsZero() const
{
	return ( (x==0.0f) && (y==0.0f) && (z==0.0f) );
}

bool Vector::IsNearlyZero() const
{
	return ( ::IsNearlyZero(x) && ::IsNearlyZero(y) && ::IsNearlyZero(z) );
}

//-------------------------------------------------
// K
//-------------------------------------------------
Vector & Vector::Normalize()
{

	float m = Length();
	
    if( m > 0.0f ) {
		m = 1.0f/m;
	} else {
		m = 0.0f;
	}
    x *= m;
    y *= m;
    z *= m;

    return *this;
}

//-------------------------------------------------
// Debug
//-------------------------------------------------
void Vector::Print()
{
	PRINTF( "Vector( %f, %f, %f, %f )\n", x,y,z,w );
}

//====================================================================
// ֐`
//====================================================================
//--------------------------------------------------------------------
// K
//--------------------------------------------------------------------
Vector Normalized( const Vector &v )
{
	Vector ret(v);
	return ret.Normalize();
}

//--------------------------------------------------------------------
// 
//--------------------------------------------------------------------
float  DotProduct( const Vector &a, const Vector &b)
{
	return a.x*b.x + a.y*b.y + a.z*b.z;
}

//--------------------------------------------------------------------
// px
//--------------------------------------------------------------------
float GetAngle( const Vector &a, const Vector &b)
{
	return acos( GetCosTheta(a,b) );
}

//--------------------------------------------------------------------
// cos
//--------------------------------------------------------------------
float GetCosTheta( const Vector &a, const Vector &b)
{
	return DotProduct(a,b)/( a.Length()*b.Length() );
}

//--------------------------------------------------------------------
// O()
//--------------------------------------------------------------------
Vector CrossProduct( const Vector &a, const Vector &b)
{
	 return Vector(	a.y*b.z - a.z*b.y,
				    a.z*b.x - a.x*b.z,
					a.x*b.y - a.y*b.x );
}

//--------------------------------------------------------------------
// Swap
//--------------------------------------------------------------------
void   SwapVector( Vector &a, Vector &b )
{
	Vector t(a);
	t.x = a.x;
	t.y = a.y;
	t.z = a.z;

	a.x = b.x;
	a.y = b.y;
	a.z = b.z;

	b.x = a.x;
	b.y = a.y;
	b.z = a.z;
}

//--------------------------------------------------------------------
// jA[CR[
//--------------------------------------------------------------------
bool   NearlyEquals( const Vector &a, const Vector &b, float r )
{
	return ( fabsf(a.x-b.x)<r && fabsf(a.y-b.y)<r && fabsf(a.z-b.z)<r );
}

//--------------------------------------------------------------------
//  returns positive if v2 is clockwise of v1,
//  minus if anticlockwise (Y axis pointing down, X axis to right)
//--------------------------------------------------------------------
enum {clockwise = 1, anticlockwise = -1};
int Sign( const Vector & v1, const Vector & v2 )
{
	if (v1.z*v2.x > v1.x*v2.z) {
		return anticlockwise;
	} else {
		return clockwise;
	}
}
