/** @file vector.h
 *  @brief VectorNXwb_
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.20
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */
#ifndef _VECTOR_H_
#define _VECTOR_H_

#include "include.h"
#include "mathlib.h"

/**
 *	  @brief VectorNX
 *
 *    xNgZNX ( x,y,z,w float̂ݑΉ )
 */
class Vector
{
public:
	//--------------------------------------------------
	// o
	//--------------------------------------------------
	float x, y, z, w;

	//-------------------------------------------------
	// o֐
	//-------------------------------------------------
public:
	Vector();
	~Vector();

	Vector( float fx, float fy, float fz = 0.0f, float fw = 1.0f );
	Vector( const Vector & v );

	//-------------------------------------------------
	// Zq
	//-------------------------------------------------
	// zCfbNX
	float		& operator [] ( u32 i );
	const float	& operator [] ( u32 i ) const;
		
	// Z
	Vector & operator += ( const Vector &v );
	Vector & operator -= ( const Vector &v );
	
	
	// XJ[{
	Vector & operator *= ( float f );
	Vector & operator /= ( float f );
	
public:
	void     Set( float x=0.0f, float y=0.0f, float z=0.0f, float w=1.0f );	// lZbg
	
	float    Length() const;		// 
	float    LengthSqr() const;		// ̂Q
	float    LengthXZ() const;		// (XZ)
	bool     IsZero() const;		// [
	bool     IsNearlyZero() const;	// jA[[
	Vector & Normalize();			// K

	//-------------------------------------------------
	// Debug
	//-------------------------------------------------
	void     Print();
};

//====================================================================
// 񃁃o֐
//====================================================================

Vector Normalized( const Vector &v );					// K
float  DotProduct( const Vector &a, const Vector &b);	// 
Vector CrossProduct( const Vector &a, const Vector &b);	// O
Vector GetProjection( const Vector &a, const Vector &b);// exNg
void   SwapVector( Vector &a, Vector &b );				// Swap
bool   NearlyEquals( const Vector &a, const Vector &b, float r );
float  GetAngle( const Vector &a, const Vector &b );		// px
float  GetCosTheta( const Vector &a, const Vector &b );	// cos
int Sign( const Vector & v1, const Vector & v2 );
// Ȃ̔
bool operator == ( const Vector &a, const Vector &b );
bool operator != ( const Vector &a, const Vector &b );

// ]
Vector operator - ( const Vector &a );
Vector operator + ( const Vector &a, const Vector &b );
Vector operator - ( const Vector &a, const Vector &b );
Vector operator * ( const Vector &v, float f );
Vector operator * ( float f, const Vector &v );
Vector operator / ( const Vector &v, float f );

// 
float operator * ( const Vector &a, const Vector &b );

//====================================================================
// 񃁃o֐
//====================================================================
static const Vector ZERO_VECTOR( 0.0f, 0.0f, 0.0f, 1.0f );

#endif //_VECTOR_H_
