/** @file vector2d.cpp
 *  @brief Vector2DNXCvg
 *
 *  @author yasufuku
 *  @date 2011/03
 *  @version 0.20
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */

//====================================================================
// CN[h
//====================================================================
#include "vector2d.h"

//====================================================================
// Vector2DNXo֐`
//====================================================================
//--------------------------------------------------------------------
// RXgN^
//--------------------------------------------------------------------
Vector2D::Vector2D()
:
x(0.0f),
y(0.0f)
{
	
}

Vector2D::Vector2D( float fx, float fy )
:
x(fx),
y(fy)
{
}

Vector2D::Vector2D( const Vector2D & v )
:
x(v.x),
y(v.y)
{
}

//--------------------------------------------------------------------
// fXgN^
//--------------------------------------------------------------------
Vector2D::~Vector2D()
{
}

//--------------------------------------------------------------------
// zCfbNX
//--------------------------------------------------------------------
float & Vector2D::operator [] ( u32 i )
{
	return(i == 0) ? x : y;
}

const float	& Vector2D::operator [] ( u32 i ) const
{
	return(i == 0) ? x : y;
}

//--------------------------------------------------------------------
// Ȃ̔
//--------------------------------------------------------------------
bool operator == ( const Vector2D &a, const Vector2D &b )
{
	return ( (a.x == b.x) && (a.y == b.y) );
}

bool operator != ( const Vector2D &a, const Vector2D &b )
{
	return( (a.x != b.x) || (a.y != b.y) );
}

//--------------------------------------------------------------------
// ]
//--------------------------------------------------------------------
Vector2D operator - ( const Vector2D &a )
{
	return Vector2D( -a.x, -a.y );
}

//--------------------------------------------------------------------
// Z
//--------------------------------------------------------------------
Vector2D & Vector2D::operator += ( const Vector2D &v )
{
	x += v.x;
    y += v.y;

    return * this;
}

Vector2D & Vector2D::operator -= ( const Vector2D &v )
{
	x -= v.x;
    y -= v.y;

    return * this;
}

Vector2D operator + ( const Vector2D &a, const Vector2D &b )
{
	Vector2D ret(a);
	ret += b;
	return ret;
}

Vector2D operator - ( const Vector2D &a, const Vector2D &b )
{
	Vector2D ret(a);
	ret -= b;
	return ret;
}

//--------------------------------------------------------------------
// XJ[{
//--------------------------------------------------------------------
Vector2D & Vector2D::operator *= ( float f )
{
	x *= f;
    y *= f;

    return * this;
}

Vector2D & Vector2D::operator /= ( float f )
{
	x /= f;
    y /= f;

    return * this;
}

Vector2D operator * ( const Vector2D &v, float f )
{
	return Vector2D( f*v.x, f*v.y );
}

Vector2D operator * ( float f, const Vector2D &v )
{
	return Vector2D( f*v.x, f*v.y );
}

Vector2D operator / ( const Vector2D &v, float f )
{
	return Vector2D( v.x/f, v.y/f );
}

//--------------------------------------------------------------------
// 
//--------------------------------------------------------------------
float operator * ( const Vector2D &a, const Vector2D &b )
{
	return a.x*b.x + a.y*b.y;
}

//-------------------------------------------------
// lZbg
//-------------------------------------------------
void Vector2D::Set( float fx, float fy )
{
	x = fx;
	y = fy;
}

//-------------------------------------------------
// 擾
//-------------------------------------------------
float Vector2D::Length() const
{
	return sqrtf( x*x + y*y );
}

//-------------------------------------------------
// ̂Q擾
//-------------------------------------------------
float Vector2D::LengthSqr() const
{
	return ( x*x + y*y );
}

//-------------------------------------------------
// [
//-------------------------------------------------
bool Vector2D::IsZero() const
{
	return ( (x==0.0f) && (y==0.0f) );
}

bool Vector2D::IsNearlyZero() const
{
	return ( ::IsNearlyZero(x) && ::IsNearlyZero(y) );
}

//-------------------------------------------------
// K
//-------------------------------------------------
Vector2D & Vector2D::Normalize()
{
	float m = Length();
	
    if( m > 0.0f ) {
		m = 1.0f/m;
	} else {
		m = 0.0f;
	}
    x *= m;
    y *= m;

    return *this;
}

//-------------------------------------------------
// Debug
//-------------------------------------------------
void Vector2D::Print()
{
	PRINTF( "Vector2D( %f, %f )\n", x,y );
}

//====================================================================
// ֐`
//====================================================================
//--------------------------------------------------------------------
// K
//--------------------------------------------------------------------
Vector2D Normalized( const Vector2D &v )
{
	Vector2D ret(v);
	return ret.Normalize();
}

//--------------------------------------------------------------------
// 
//--------------------------------------------------------------------
float  DotProduct( const Vector2D &a, const Vector2D &b)
{
	return a.x*b.x + a.y*b.y;
}

//--------------------------------------------------------------------
// px
//--------------------------------------------------------------------
float GetAngle( const Vector2D &a, const Vector2D &b)
{
	return acos( GetCosTheta(a,b) );
}

//--------------------------------------------------------------------
// cos
//--------------------------------------------------------------------
float GetCosTheta( const Vector2D &a, const Vector2D &b)
{
	return DotProduct(a,b)/( a.Length()*b.Length() );
}

//--------------------------------------------------------------------
// Swap
//--------------------------------------------------------------------
void   SwapVector2D( Vector2D &a, Vector2D &b )
{
	Vector2D t(a);
	t.x = a.x;
	t.y = a.y;

	a.x = b.x;
	a.y = b.y;

	b.x = a.x;
	b.y = a.y;
}

//--------------------------------------------------------------------
// jA[CR[
//--------------------------------------------------------------------
bool   NearlyEquals( const Vector2D &a, const Vector2D &b, float r )
{
	return ( fabsf(a.x-b.x)<r && fabsf(a.y-b.y)<r );
}

//--------------------------------------------------------------------
//  returns positive if v2 is clockwise of v1,
//  minus if anticlockwise (Y axis pointing down, X axis to right)
//--------------------------------------------------------------------
enum {clockwise = 1, anticlockwise = -1};
int Sign( const Vector2D & v1, const Vector2D & v2 )
{
	if (v1.y*v2.x > v1.x*v2.y) {
		return anticlockwise;
	} else {
		return clockwise;
	}
}
