/** @file keyboard.h
 *  @brief KeyboardNXwb_
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.10
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */
#ifndef _KEYBOARD_H_
#define _KEYBOARD_H_

#include "include.h"
#include "task.h"

enum eSKey {
	KEY_SHIFT= BIT0,
	KEY_ALT	 = BIT1,
	KEY_CTRL = BIT2,
	KEY_ESC	 = BIT3,
	KEY_F1   = BIT4,
};

enum eKey {
	KEY_A		= BIT0,
	KEY_B		= BIT1,
	KEY_C		= BIT2,
	KEY_D		= BIT3,
	KEY_E		= BIT4,
	KEY_F		= BIT5,
	KEY_G		= BIT6,
	KEY_H		= BIT7,
	KEY_I		= BIT8,
	KEY_J		= BIT9,
	KEY_K		= BIT10,
	KEY_L		= BIT11,
	KEY_M		= BIT12,
	KEY_N		= BIT13,
	KEY_O		= BIT14,
	KEY_P		= BIT15,
	KEY_Q		= BIT16,
	KEY_R		= BIT17,
	KEY_S		= BIT18,
	KEY_T		= BIT19,
	KEY_U		= BIT20,
	KEY_V		= BIT21,
	KEY_W		= BIT22,
	KEY_X		= BIT23,
	KEY_Y		= BIT24,
	KEY_Z		= BIT25,
	KEY_UP		= BIT26,
	KEY_DOWN	= BIT27,
	KEY_RIGHT	= BIT28,
	KEY_LEFT	= BIT29,
	KEY_SPACE   = BIT30,
	KEY_ENTER   = BIT31,
};


/**
 *  @brief KeyboardNX
 *
 *	L[o[hNXiL[{[hRg[Ɏgpj
 */
class Keyboard : public Task
{
private:
	static Keyboard * s_pSelf;
	static u32 s_unInputKey;
	static u32 s_unInputSpecialKey;
	u32		m_unPress;
	u32		m_unTrigger;
	u32		m_unSPress;
	u32		m_unSTrigger;
	
protected:
	Keyboard();
	virtual ~Keyboard();

	//----------------------------------------------
	// bZ[W
	//----------------------------------------------
	virtual int OnInit();		// 
	virtual int OnMain();		// ^XNC

public:
	static Keyboard * Create( Task * pParent );
	static Keyboard * GetSelf(){ return s_pSelf; }
	
	static bool Press( eKey key );
	static bool Trigger( eKey key );
	static bool Press( eSKey skey );
	static bool Trigger( eSKey skey );
	//static bool Press( eKey key, eSKey skey );
	//static bool Trigger( eKey key, eSKey skey );

	void CallBackPressKey( u8 key, int x, int y );
	void CallBackUnpressKey( u8 key, int x, int y );
	void CallBackPressSpecialKey( int key, int x, int y );
	void CallBackUnpressSpecialKey( int key, int x, int y );
	void CallBackCheckModifiers( int mod );	
};

#endif //_KEYBOARD_H_
